#!/bin/bash
#fglrx-kernel-module_sfs_builder_for_Puppy_v0.1
#This is a fglrx kernel module driver builder for Puppy and Puppy derivatives.  
#It will build fglrx.ko for kernels up to 4.19.  
#It will also install amdocl32.icd, amdocl64.icd and its libraries.  Enabling OpenCl.
#Build by enrique

srcdir=$PWD
pkgver=15.9
_amdver=15.201.1151
_archdir=$(uname -m)
OUTPUTDIR="${srcdir}/fglrx-driver-module"
CATALYSTS="AMD-Catalyst-${pkgver}-Linux-installer-${_amdver}-x86.x86_64.run"
OPENCL_LIB_64="usr/lib/x86_64-linux-gnu"
OPENCL_LIB_32="usr/lib/i386-linux-gnu"


clean(){
	echo "*** Cleaning old files and directories ***"
	rm -f ${srcdir}/fglrx-driver-module.squashfs
	rm -rf ${srcdir}/extracted_files
	rm -rf ${OUTPUTDIR}
}

extractpackage() {
	if [[ -f $CATALYSTS ]]; then
		echo "*** Extracting $CATALYSTS ***"
		/bin/sh ./AMD-Catalyst-${pkgver}-Linux-installer-${_amdver}-x86.x86_64.run --extract extracted_files
	else
		echo "*** Error $CATALYSTS not found ***"
		echo "*** Please download package, try and unzip in this folder ***"
		echo "*** Try downloading from AMD.  You can use the following instruction to try ***"
		echo ""
		echo "wget --referer='http://support.amd.com/en-us/download/desktop?os=Linux+x86' \\"
		echo "https://www2.ati.com/drivers/linux/amd-catalyst-15.9-linux-installer-15.201.1151-x86.x86_64.zip"
		echo ""		
		exit 1
	fi
}

patchpackage() {
	echo "*** Patching Sources ***"
	# Patching	
	OsName="`uname -r`"
	OsRelease=$OsName
    major=${OsRelease%%.*}
    kernel_release_rest=${OsRelease#*.}
    minor=${kernel_release_rest%%-*}
    minor=${minor%%.*}
    echo "minor=$minor"
    if [ $major -ge 4 ]; then
		if [ $minor -ge 2 ]; then
#   		patch -Np1 -i ../makefile_compat.patch
			patch -Np1 -i ../lano1106_fglrx_intel_iommu.patch
			patch -Np1 -i ../lano1106_kcl_agp_13_4.patch
			patch -Np1 -i ../4.2-amd-from_crimson_15.11.patch
		fi
		if [ $minor -ge 3 ]; then
			patch -Np1 -i ../4.3-kolasa-seq_printf.patch
			patch -Np1 -i ../4.3-gentoo-mtrr.patch
			patch -Np1 -i ../fglrx_gpl_symbol.patch
			if [ ${_archdir} != x86_64 ]; then
				echo "_archdir=${_archdir}"
				patch -Np1 -i ../crimson_i686_xg.patch
			fi
		fi
		if [ $minor -ge 4 ]; then		
			patch -Np1 -i ../4.4-manjaro-xstate.patch
			patch -Np1 -i ../grsec_arch.patch
		fi
		if [ $minor -ge 6 ]; then
			patch -Np1 -i ../4.6-arch-get_user_pages-page_cache_release.patch
			patch -Np1 -i ../makesh-dont-check-gcc-version.patch
		fi
		if [ $minor -ge 7 ]; then
			patch -Np1 -i ../4.7-arch-cpu_has_pge-v2.patch
		fi
		if [ $minor -ge 9 ]; then
			patch -Np1 -i ../4.9_over_4.6-arch-get_user_pages_remote.patch
		fi
		if [ $minor -ge 10 ]; then
			patch -Np1 -i ../4.10-arch-sling00-virtual_address-acpi_get_table_with_size.patch
		fi
		if [ $minor -ge 11 ]; then
			patch -Np1 -i ../4.11-npfeiler-signal_vmf.patch
		fi
		if [ $minor -ge 12 ]; then
			patch -Np1 -i ../4.12-npfeiler-PUD_OFFSET.patch
			patch -Np1 -i ../4.12-arch-remove_clts.patch
			patch -Np1 -i ../4.12-npfeiler-movsl_mask.patch
		fi
		if [ $minor -ge 13 ]; then
			patch -Np1 -i ../4.13-npfeiler-wait_queue_t.patch
		fi	
		if [ $minor -ge 14 ]; then
			patch -Np1 -i ../4.14-npfeiler-task_struct-mm_segment_t.patch
			patch -Np1 -i ../4.14.21_4.15.5-npfeiler-flush_tlb_one_kernel.patch
		fi
		if [ $minor -ge 17 ]; then
			patch -Np1 -i ../4.17-pci_get_bus_and_slot_deprecated.patch
		fi	
		if [ $minor -ge 19 ]; then
			patch -Np1 -i ../4.19-__kernel_fpu_begin__kernel_fpu_end_deprecated.patch
		fi
	fi
}

build() {
	echo "*** Creating directory structure ***"

	# Prepare modules source files
	mkdir -p ${OUTPUTDIR}/usr/src/fglrx-${pkgver}/2.6.x/
	mkdir -p ${OUTPUTDIR}/lib/modules/$(uname -r)/kernel/drivers/char/dri/
	mkdir -p ${OUTPUTDIR}/etc/OpenCL/vendors/
	mkdir -p ${OUTPUTDIR}/usr/bin/
	mkdir -p ${OUTPUTDIR}/${OPENCL_LIB_64}
	mkdir -p ${OUTPUTDIR}/${OPENCL_LIB_64}/dri/
	mkdir -p ${OUTPUTDIR}/${OPENCL_LIB_32}
	mkdir -p ${OUTPUTDIR}/${OPENCL_LIB_32}/dri/
	mkdir -p ${OUTPUTDIR}/usr/sbin/
	mkdir -p ${OUTPUTDIR}/etc/profile.d/
	
	if [ ${_archdir} != x86_64 ]; then
		mkdir -p arch/$(uname -m)/lib/modules/fglrx/build_mod/
		cp -f arch/x86/lib/modules/fglrx/build_mod/libfglrx_ip.a arch/$(uname -m)/lib/modules/fglrx/build_mod/libfglrx_ip.a	
	fi
	
	echo "*** Copying files ***"
	cp common/lib/modules/fglrx/build_mod/*.c  ${OUTPUTDIR}/usr/src/fglrx-${pkgver}
	cp common/lib/modules/fglrx/build_mod/*.h  ${OUTPUTDIR}/usr/src/fglrx-${pkgver}
	cp common/lib/modules/fglrx/build_mod/2.6.x/Makefile ${OUTPUTDIR}/usr/src/fglrx-${pkgver}/2.6.x
	cp common/lib/modules/fglrx/build_mod/make.sh ${OUTPUTDIR}/usr/src/fglrx-${pkgver}
	cp arch/${_archdir}/lib/modules/fglrx/build_mod/libfglrx_ip.a ${OUTPUTDIR}/usr/src/fglrx-${pkgver}
	
    ## OpenCL sdk files
	if [ ${_archdir} != x86_64 ]; then
		cp arch/x86/usr/bin/clinfo						${OUTPUTDIR}/usr/bin/clinfo	
	else
		cp arch/x86_64/usr/bin/clinfo					${OUTPUTDIR}/usr/bin/clinfo
		cp arch/x86_64/etc/OpenCL/vendors/amdocl64.icd  ${OUTPUTDIR}/etc/OpenCL/vendors/amdocl64.icd
#		cp -R arch/x86_64/usr/lib64/libamdocl*.*     	${OUTPUTDIR}/${OPENCL_LIB_64}
#		cp -R arch/x86_64/usr/lib64/libOpenCL*.*     	${OUTPUTDIR}/${OPENCL_LIB_64}
		cp arch/x86_64/usr/lib64/lib*.*     			${OUTPUTDIR}/${OPENCL_LIB_64}
		cp arch/x86_64/usr/X11R6/lib64/lib*.*     		${OUTPUTDIR}/${OPENCL_LIB_64}
		cp arch/x86_64/usr/X11R6/lib64/modules/dri/fglrx_dri.so    	${OUTPUTDIR}/${OPENCL_LIB_64}/dri/
	fi
	cp arch/x86/etc/OpenCL/vendors/amdocl32.icd         ${OUTPUTDIR}/etc/OpenCL/vendors/amdocl32.icd
#	cp -R arch/x86/usr/lib/libamdocl*.*          		${OUTPUTDIR}/${OPENCL_LIB_32}
#	cp -R arch/x86/usr/lib/libOpenCL*.*          		${OUTPUTDIR}/${OPENCL_LIB_32}
	cp arch/x86/usr/lib/lib*.*          				${OUTPUTDIR}/${OPENCL_LIB_32} 
	cp arch/x86/usr/X11R6/lib/lib*.*     				${OUTPUTDIR}/${OPENCL_LIB_32}
	cp arch/x86/usr/X11R6/lib/modules/dri/fglrx_dri.so     			${OUTPUTDIR}/${OPENCL_LIB_32}/dri/
	cp common/usr/sbin/atigetsysteminfo.sh				${OUTPUTDIR}/usr/sbin/
	
    
#./extracted_files/arch/x86_64/usr/X11R6/lib64/libXvBAW.so.1.0
#./extracted_files/arch/x86/usr/X11R6/lib/libXvBAW.so.1.0
#ln -s
#./extracted_files/arch/x86_64/usr/lib64/libaticalcl.so
#./extracted_files/arch/x86/usr/lib/libaticalcl.so
#./extracted_files/arch/x86_64/usr/lib64/libatiuki.so.1.0
#./extracted_files/arch/x86/usr/lib/libatiuki.so.1.0

    
    
	### blacklisting - required for disabling radeon on systems with KMS ###                
	mkdir -p ${OUTPUTDIR}/etc/modprobe.d
	blacklistfile="${OUTPUTDIR}/etc/modprobe.d/blacklist-fglrx.conf"
	echo "# Advanced Micro Devices, Inc."> ${blacklistfile}
	echo "# radeon conflicts with AMD Linux Graphics Driver" >> ${blacklistfile}
	echo "blacklist radeon" >> ${blacklistfile}
	
ATI_PROFILE="### START ATI FGLRX ###
### Automatically modified by ATI Proprietary driver scripts
### Please do not modify between START ATI FGLRX and END ATI FGLRX

#setting LD_LIBRARY_PATH is not required for ATI FGLRX
#if [ $LD_LIBRARY_PATH ]
#then
#  if ! set | grep LD_LIBRARY_PATH | grep /usr/lib/i386-linux-gnu:/usr/lib64 > /dev/null
#  then
    #LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/usr/lib/i386-linux-gnu:/usr/lib64       
    #export LD_LIBRARY_PATH
#  fi
#else 
  #LD_LIBRARY_PATH=/usr/lib/i386-linux-gnu:/usr/lib64
  #export LD_LIBRARY_PATH
#fi

if [ $LIBGL_DRIVERS_PATH ]
then
  if ! set | grep LIBGL_DRIVERS_PATH | grep /usr/lib/i386-linux-gnu/dri:/usr/lib/x86_64-linux-gnu/dri > /dev/null
  then
    LIBGL_DRIVERS_PATH=$LIBGL_DRIVERS_PATH:/usr/lib/i386-linux-gnu/dri:/usr/lib/x86_64-linux-gnu/dri
    export LIBGL_DRIVERS_PATH
  fi
else
  LIBGL_DRIVERS_PATH=/usr/lib/i386-linux-gnu/dri:/usr/lib/x86_64-linux-gnu/dri
  export LIBGL_DRIVERS_PATH
fi

### END ATI FGLRX ###
"
	printf "${ATI_PROFILE}" > ${OUTPUTDIR}/etc/profile.d/ati-fglrx.sh
	chmod +x ${OUTPUTDIR}/etc/profile.d/ati-fglrx.sh
	
	cp arch/${_archdir}/lib/modules/fglrx/build_mod/libfglrx_ip.a common/lib/modules/fglrx/build_mod
	cd ${srcdir}/extracted_files/common/lib/modules/fglrx/build_mod
	./make.sh
	if [ "$?" -eq 0 ]; then 
		cp ${srcdir}/extracted_files/common/lib/modules/fglrx/build_mod/2.6.x/fglrx.ko ${OUTPUTDIR}/lib/modules/$(uname -r)/kernel/drivers/char/dri/
		mkdir -p /lib/modules/$(uname -r)/kernel/drivers/char/dri
		ln -sf ${OUTPUTDIR}/lib/modules/$(uname -r)/kernel/drivers/char/dri/fglrx.ko /lib/modules/$(uname -r)/kernel/drivers/char/dri/fglrx.ko
		depmod 
		cp /lib/modules/$(uname -r)/modules* ${OUTPUTDIR}/lib/modules/$(uname -r)/
		rm /lib/modules/$(uname -r)/kernel/drivers/char/dri/fglrx.ko
	else
		echo ""
		echo "*** Error - fglrx.ko compilation failed ***"
		echo ""
		exit 1
	fi
	cd ${srcdir}
	mksquashfs ${OUTPUTDIR} ${OUTPUTDIR}.squashfs -comp xz
}


### MAIN ###
clean
extractpackage
cd ${srcdir}/extracted_files
patchpackage
build
echo "*** Build Complete - Squash is ready ***"
exit 0
